@@ -1,7 +1,7 @@
 /* -*- mode: C; c-file-style: "gnu" -*- */
 /*
  * Copyright (C) 2003 Richard Hult <richard@imendio.com>
- * Copyright (C) 2003 Johan Dahlin <jdahlin@gnome.org>
+ * Copyright (C) 2003 Johan Dahlin <johan@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +29,8 @@
 
 static void expr_clear (Expr *expr);
 
+static time_t now = 0;
+
 const char *
 operator_to_string (ExprOp op)
 {
@@ -549,6 +551,27 @@ constant_free (Constant *constant)
   g_free (constant);
 }
 
+static const char *
+constant_get_string_value_const (Constant *constant)
+{
+  switch (constant->type)
+    {
+    case CONSTANT_TYPE_STRING:
+      return constant->v.str;
+
+    case CONSTANT_TYPE_RANGE:
+    case CONSTANT_TYPE_NONE:
+    case CONSTANT_TYPE_DATE:
+    case CONSTANT_TYPE_SIZE:
+    case CONSTANT_TYPE_INT:
+      g_warning ("Trying to get const string value for constant type '%s'.",
+		 constant_type_to_string (constant->type));
+      return "";
+    }
+
+  return "";
+}
+
 char *
 constant_get_string_value (Constant *constant)
 {
@@ -633,6 +656,45 @@ unit_from_string (const char * str)
     }
 }
 
+static const char *
+variable_get_string_value_const (Variable variable, Song *song)
+{
+  const char *tmp;
+  
+  switch (variable)
+    {
+    case VARIABLE_TITLE:
+      return string_entry_get_str (song->title);
+    case VARIABLE_ARTIST:
+      return string_entry_get_str (song->artist);
+    case VARIABLE_ALBUM:
+      return string_entry_get_str (song->album);
+    case VARIABLE_GENRE:
+      if (song->genre == -1)
+	return "";
+
+      tmp = id3_genres[song->genre];
+      if (!tmp)
+	tmp = "";
+      
+      return tmp;
+      
+    case VARIABLE_YEAR:
+    case VARIABLE_TRACK_NUMBER:
+    case VARIABLE_SIZE:
+    case VARIABLE_BIT_RATE:
+    case VARIABLE_SAMPLE_RATE:
+    case VARIABLE_LENGTH:
+    case VARIABLE_PLAY_COUNT:
+    case VARIABLE_LAST_PLAYED:
+    case VARIABLE_RATING:
+    default:
+      g_warning ("Can't get string for variable %s.", variable_to_string (variable));
+      return "";
+    }
+}
+
+#if 0
 static char *
 variable_get_string_value (Variable variable, Song *song)
 {
@@ -683,6 +745,7 @@ variable_get_string_value (Variable variable, Song *song)
       return NULL;
     }
 }
+#endif
 
 int
 constant_get_int_value (Constant *constant)
@@ -760,22 +823,29 @@ variable_get_int_value (Variable variable, Song *song)
     }
 }
 
+static gboolean
+get_now (gpointer data)
+{
+  now = time (NULL);
+  return TRUE;
+}
+
 time_t
 constant_get_date_value (Constant *constant)
 {
-  time_t now = time (NULL);
   int list[] = { 60, 3600, 86400, 86400*7, 86400*30, 86400*365 };
   int unit = constant_get_unit (constant);
 
+  if (G_UNLIKELY (now == 0))
+    {
+      g_timeout_add (60*1000, get_now, NULL);
+      get_now (NULL);
+    }
+  
   return now - (time_t) constant_get_int_value (constant) * list[unit];
 }
 
-static time_t
-variable_get_date_value (Variable variable, Song *song)
-{
-  return (time_t) variable_get_int_value (variable, song);
-}
-
+#define variable_get_date_value(v,s) ((time_t) variable_get_int_value (v, s))
 
 /*
  * Expression evaluation.
@@ -883,7 +953,8 @@ evaluate_comparison (Expr *e1, Expr *e2, ExprOp op, Song *song)
 {
   Constant *constant;
   Variable variable;
-  char *str1, *str2;
+  const char *str1;
+  const char *str2;
   int i1, i2;
   int start, end;
   gboolean ret;
@@ -928,14 +999,11 @@ evaluate_comparison (Expr *e1, Expr *e2, ExprOp op, Song *song)
   switch (constant->type)
     {
     case CONSTANT_TYPE_STRING:
-      str1 = variable_get_string_value (variable, song);
-      str2 = constant_get_string_value (constant);
+      str1 = variable_get_string_value_const (variable, song);
+      str2 = constant_get_string_value_const (constant);
       
       ret = string_compare_with_op (str1, str2, op);
       
-      g_free (str1);
-      g_free (str2);
-
       return ret;
 
     case CONSTANT_TYPE_INT:
@@ -973,7 +1041,7 @@ evaluate_binary_op (Expr *expr, Song *song)
   Expr *e1, *e2;
   gboolean v1, v2;
 
-  g_assert (is_binary (expr->op));
+  /*g_assert (is_binary (expr->op));*/
   
   e1 = expr->v.binary.expr_1;
   e2 = expr->v.binary.expr_2;
