@@ -285,7 +285,7 @@ start_element_handler (GMarkupParseContext  *context,
 			 element);
 	      break;
 	    }
-	  
+
 	  data->query = query_new (NULL);
 	  
 	  push_state (data, STATE_QUERY);
@@ -516,6 +516,9 @@ end_element_handler (GMarkupParseContext  *context,
 	  query_set_expr (data->query, expr);
 	}
       playlist_set_query (data->playlist, data->query);
+
+      query_unref (data->query);
+      data->query = NULL;
       
       break;
 
@@ -585,9 +588,7 @@ playlists_from_xml (const char *str)
   data.context = g_markup_parse_context_new (data.parser, 0, &data, NULL);
   
   if (!g_markup_parse_context_parse (data.context, str, -1, NULL))
-    {
-      data.playlists = NULL;
-    }
+    data.playlists = NULL;
   
   g_markup_parse_context_free (data.context);
   g_free (data.parser);
@@ -790,7 +791,7 @@ playlist_xml_save_playlists (GList *playlists)
 GList *
 playlist_xml_load_playlists (void)
 {
-  const char *filename;
+  char *filename;
   char *str;
   static GList *playlists = NULL;
 
@@ -803,12 +804,12 @@ playlist_xml_load_playlists (void)
 			       "playlists.xml",
 			       NULL);
   
-  if (!g_file_get_contents (filename, &str, NULL, NULL))
-    return NULL;
+  if (g_file_get_contents (filename, &str, NULL, NULL))
+    playlists = playlists_from_xml (str);
 
-  playlists = playlists_from_xml (str);
   g_free (str);
-
+  g_free (filename);
+  
   return playlists;
 }
 
